# -*- coding: utf-8 -*-
# Copyright 2022 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Turbinia API client command-line tool."""

from typing import Tuple, Sequence


def generate_option_parameters(
    option_name: str) -> Tuple[Tuple[Sequence[str], str], dict]:
  """Builds click.Option or click.Command arguments based on a given parameter 
      name.
  """
  return ((['--' + option_name], option_name), {'required': False, 'type': str})
